-- Product prefix: pd_ systemid: 2
-- Tables for the Product Module
-- Copyright (c) 2001 The FreeMoney Project
-- This work is released under the GPL
-- $Id: create_tables_product.sql,v 1.27 2001/06/24 09:57:10 kaare Exp $

-- Sequences
DROP SEQUENCE pd_artsequence;

CREATE SEQUENCE pd_artsequence INCREMENT 1;

--Tables
DROP TABLE pd_deliverytype;
DROP TABLE pd_article;
DROP TABLE pd_artprice;
DROP TABLE pd_inventory;
DROP TABLE pd_arthist;

create table pd_deliverytype (
       type                 integer not null,
       name                 varchar(128),
       form                 varchar(128),
       orderstatus          integer,
       updated              timestamp not null default 'now',
 primary key (type)
);

CREATE TABLE pd_article (
       art_id               integer NOT NULL DEFAULT nextval('pd_artsequence'),
       pgrp_id              integer,
       articlenr            varchar(20),
       altartnr             varchar(20),
       description          varchar(128),
       unitweight           decimal,
       delivery             integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (art_id ),
 FOREIGN KEY (pgrp_id )     REFERENCES pd_productgroup,
 FOREIGN KEY (delivery)     REFERENCES pd_deliverytype
);

CREATE TABLE pd_artprice (
       art_id               integer NOT NULL,
       type                 integer,
       price                decimal,
       validfrom            timestamp,
       validto              timestamp,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (art_id ,type,validfrom),
 FOREIGN KEY (art_id)   REFERENCES pd_article
  ON DELETE CASCADE
  ON UPDATE CASCADE,
 FOREIGN KEY (type)         REFERENCES sy_typetext
);

CREATE TABLE pd_inventory (
       art_id               integer NOT NULL,
--       placement_id         integer,
       unitcost             decimal,
       stockqty             decimal,
       minqty               decimal,
       maxqty               decimal,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (art_id ),
 FOREIGN KEY (art_id )   REFERENCES pd_article
);

CREATE TABLE pd_arthist (
       art_id               integer NOT NULL,
       type                 integer,
       period               date,
       qty                  decimal,
       revenue              decimal,
       cost                 decimal,
       stock                decimal,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (art_id ),
 FOREIGN KEY (art_id )   REFERENCES pd_article
);

-- Views
DROP VIEW pd_artpricecurr_v;

 CREATE VIEW pd_artpricecurr_v AS
    SELECT a.art_id , a.articlenr, a.altartnr, a.description, a.unitweight,
           p.validfrom, p.validto, p.price
    FROM   pd_article a, pd_artprice p
    WHERE  a.art_id  = p.art_id 
    AND    p.type = 110001
    AND    p.validfrom <= timenow()
    AND    p.validto > timenow();

