-- Sales Order prefix: so_ systemid: 3
-- Tables for the Sales Order Module
-- Copyright (c) 2001 The Freemoney Project
-- This work is released under the GPL
-- $Id: create_tables_so.sql,v 1.16 2001/06/24 09:57:10 kaare Exp $

-- Sequences
DROP SEQUENCE so_ordersequence;
DROP SEQUENCE so_invoicesequence;
DROP SEQUENCE so_invbatchsequence;
DROP SEQUENCE so_invtplsequence;
DROP SEQUENCE so_invtplcsequence;

CREATE SEQUENCE so_ordersequence   INCREMENT 1;
CREATE SEQUENCE so_invoicesequence INCREMENT 1;
CREATE SEQUENCE so_invbatchsequence INCREMENT 1;
CREATE SEQUENCE so_invtplsequence INCREMENT 1;
CREATE SEQUENCE so_invtplcsequence INCREMENT 1;

--Tables
DROP TABLE so_orderhead;
DROP TABLE so_orderline;
DROP TABLE so_delivery;
DROP TABLE so_invoice;
DROP TABLE so_invoicebatch;
DROP TABLE so_invoicetemplate;
DROP TABLE so_invtplchoice;

CREATE TABLE so_orderhead (
       ordernr              integer NOT NULL DEFAULT nextval('so_ordersequence'),
       contact_id           integer NOT NULL,
       status               integer,
       currency_id          integer,
       disctype             integer,
       discount             decimal,
       ourref               varchar(128),
       yourref              varchar(128),
       credit_code          integer,
       deliverycode         integer,
       paycode              integer,
       freightcode          integer,
       orderdate            timestamp DEFAULT 'now',
       deliverydate         timestamp,
       ordertotal           decimal,
       vattotal             decimal,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (ordernr),
 FOREIGN KEY (currency_id)  REFERENCES gl_currency,
 FOREIGN KEY (disctype)     REFERENCES sy_typetext,
 FOREIGN KEY (credit_code)  REFERENCES co_creditterms,
 FOREIGN KEY (deliverycode) REFERENCES co_deliveryterms,
 FOREIGN KEY (paycode)      REFERENCES co_paymentterms,
 FOREIGN KEY (freightcode)  REFERENCES co_freightcode,
 FOREIGN KEY (contact_id)   REFERENCES co_contact
);

CREATE TABLE so_orderline (
       ordernr              integer NOT NULL,
       linenr               integer NOT NULL,
       art_id               integer,
       description          varchar(128),
       qty                  decimal,
       qtyshipped           decimal,
       unitprice            decimal,
       disctype             decimal,
       discount             decimal,
       linetotal            decimal,
       linevat              decimal,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (ordernr, linenr),
 FOREIGN KEY (art_id )      REFERENCES pd_article,
 FOREIGN KEY (disctype)     REFERENCES sy_typetext,
 FOREIGN KEY (ordernr)      REFERENCES so_orderhead
  ON DELETE CASCADE
  ON UPDATE CASCADE
);

CREATE TABLE so_invoicebatch (
       batchnr              integer NOT NULL DEFAULT nextval('so_invbatchsequence'),
       ordernr              integer,
       page                 integer,
       status               integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (batchnr)
);

CREATE TABLE so_invoicetemplate (
       invtplnr             integer NOT NULL DEFAULT nextval('so_invtplsequence'),
       template             varchar(128),
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (invtplnr)
);

CREATE TABLE so_invtplchoice (
       invtplcnr            integer NOT NULL DEFAULT nextval('so_invtplcsequence'),
       invtplnr             integer,
       ordernr              integer,
       contact_id           integer,
       custgrp_id           integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (invtplcnr),
 FOREIGN KEY (invtplnr)     REFERENCES so_invoicetemplate
);

CREATE TABLE so_invoice (
       invoicenr            integer NOT NULL DEFAULT nextval('so_invoicesequence'),
       ordernr              integer NOT NULL,
       batchnr              integer NOT NULL,
       invoicetotal         decimal,
       invoicedate          timestamp,
       status               integer,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (invoicenr),
 FOREIGN KEY (ordernr)      REFERENCES so_orderhead
  ON DELETE RESTRICT
  ON UPDATE CASCADE,
 FOREIGN KEY (batchnr)     REFERENCES so_invoicebatch
);

CREATE TABLE so_delivery (
       ordernr              integer NOT NULL,
       status               integer,
       form                 varchar(20),
       tracking             varchar(128),
       weight               decimal,
       updated              timestamp NOT NULL DEFAULT 'now',
 PRIMARY KEY (ordernr),
 FOREIGN KEY (ordernr)      REFERENCES so_orderhead
  ON DELETE CASCADE
  ON UPDATE CASCADE
);

-- Views
DROP VIEW so_ordercust_v;
DROP VIEW so_delinv_v;
DROP VIEW so_currorderarticle_v ;

 CREATE VIEW so_ordercust_v AS
  SELECT o.ordernr, o.contact_id , o.ordertotal, o.status,
   (SELECT lastname||', '||firstname FROM co_person p, so_orderhead o
     WHERE p.contact_id  = o.contact_id
    UNION
     SELECT name FROM co_company c, so_orderhead o
      WHERE c.contact_id  = o.contact_id) AS name,
    a.address1, a.address2, a.address3, a.type as addresstype, a.country_id ,
    p.postalcode, p.name as city
   FROM so_orderhead o, co_customer c, co_address a, co_postalcodes p
   WHERE o.contact_id  = c.contact_id
   AND o.contact_id  = a.contact_id
   AND a.country_id  = p.country_id  AND a.postalcode = p.postalcode;

 CREATE VIEW so_delinv_v AS
    SELECT d.ordernr, d.weight, d.form, d.tracking,
           i.invoicenr, i.invoicetotal, i.status
    FROM   so_delivery d, so_invoice i
    WHERE  d.ordernr = i.ordernr;

 CREATE VIEW so_currorderarticle_v AS
    SELECT l.ordernr, l.linenr, l.description as linedesc, l.qty,
           l.unitprice as lineprice, l.linetotal,
           a.art_id, a.altartnr, a.description, a.unitweight,
           p.price
    FROM   so_orderline l, pd_article a, pd_artprice p
    WHERE  l.art_id  = a.art_id
    AND    l.art_id  = p.art_id
    AND    p.validfrom < timenow()
    AND    p.validto > timenow();

-- EOF
